@@ -82,7 +82,6 @@ point_mesh <- function(dimension = c(2,3),
     }
     coordinates <- own_coordinates
   } else if (is.null(template) && is.null(own_coordinates)) {
-    #coordinates <- diegr::HCGSN256
     template <- "HCGSN256"
    }
 
@@ -299,7 +298,6 @@ plot_point_mesh <- function(mesh,
     rgl::points3d(mesh$x, mesh$y, mesh$z, col = col, cex = cex)
 
     if (is.null(own_coordinates)) {
-      #own_coordinates <- diegr::HCGSN256$D3
       if (!is.null(sensor_select)) {
         coords_full <- diegr::HCGSN256$D3
         sensor_index <- which(coords_full$sensor %in% sensor_select)
@@ -329,7 +327,6 @@ plot_point_mesh <- function(mesh,
   else if (all(c("x", "y") %in% colnames(mesh))) {
 
     if (is.null(own_coordinates)) {
-      #own_coordinates <- diegr::HCGSN256$D2
       if (!is.null(sensor_select)) {
         coords_full <- diegr::HCGSN256$D2
         sensor_index <- which(coords_full$sensor %in% sensor_select)
@@ -443,13 +440,7 @@ plot_point_mesh <- function(mesh,
 #'  rgl::wire3d(rgl::mesh3d(M$D3$x, M$D3$y, M$D3$z, triangles = t(TRI)))
 make_triangulation <- function(mesh) {
 
-  required_cols <- c("x", "y")
-  missing_cols <- setdiff(required_cols, colnames(mesh))
-
-  if (length(missing_cols) > 0) {
-    stop(paste("The following required columns in mesh are missing:",
-               paste(missing_cols, collapse = ", ")))
-  }
+  stop_if_missing_cols(mesh, required_cols = c("x", "y"))
 
   if (!is.numeric(mesh$x) || !is.numeric(mesh$y)) {
     stop("Columns 'x' and 'y' must be numeric.")
