@@ -9,8 +9,9 @@ test_that("check boxplot_rt structure", {
 
 test_that("check boxplot_epoch structure", {
   skip_on_cran()
-  p_test <- boxplot_epoch(epochdata, amplitude = "signal", subject = 1,
-                          channel = "E34", time_lim = c(1:2))
+  edata <- epochdata |>
+    pick_data(subject_rg = 1, sensor_rg = "E34")
+  p_test <- boxplot_epoch(edata, amplitude = "signal", time_lim = c(1:2))
 
   expect_s3_class(p_test, "plotly")
 
@@ -18,8 +19,9 @@ test_that("check boxplot_epoch structure", {
 
 test_that("check boxplot_subject structure", {
   skip_on_cran()
-  p_test <- boxplot_subject(epochdata, amplitude = "signal", channel = "E34",
-                            time_lim = c(1:2))
+  edata <- epochdata |>
+    pick_data(epoch_rg = 1, sensor_rg = "E34")
+  p_test <- boxplot_subject(epochdata, amplitude = "signal", time_lim = c(1:2))
 
   expect_s3_class(p_test, "plotly")
 
@@ -27,8 +29,8 @@ test_that("check boxplot_subject structure", {
 
 test_that("check interactive_waveforms structure", {
   skip_on_cran()
-  p_test <- interactive_waveforms(epochdata, amplitude = "signal", subject = 1,
-                                  channel = "E65", t0 = 10, level = "epoch")
+  subdata <- pick_data(epochdata, subject_rg = 1, sensor_rg = "E65")
+  p_test <- interactive_waveforms(subdata, amplitude = "signal", t0 = 10, level = "epoch")
   expect_s3_class(p_test, "plotly")
 
 })
